/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "seg_pool_mgr.hh"
#include "buffer_pool.hh"
#include "chan_list.hh"
#include <stdexcept>

using namespace sends;
using namespace std;

//=====================================  Constructor
seg_pool_mgr::seg_pool_mgr(void) {
}

//=====================================  Destructor
seg_pool_mgr::~seg_pool_mgr(void) {
    long N = mPools.size();
    for (long i=0; i<N; ++i) {
	if (mPools[i]) delete mPools[i];
	mPools[i] = 0;
    }
}

//=====================================  Add a new buffer pool
void 
seg_pool_mgr::add_pool(bufclass_type id, long nbuf, long lbuf) {
    unsigned long N = mPools.size();
    if (id < N) {
	mPools.resize(id);
	for (unsigned long i=N; i<id; i++) mPools[i] = 0;
	N = mPools.size();
    }
    if (id == N) {
	mPools.push_back(new buffer_pool(nbuf, lbuf) );
    } else {
	if (mPools[id] != 0) delete mPools[id];
	mPools[id] = new buffer_pool(nbuf, lbuf);
    }
}

//======================================  Chose a buffer class
seg_pool_mgr::bufclass_type 
seg_pool_mgr::get_buffer_class(select_type selid) const {
    //if (Channel_List[selid].rate() < 256.0) return kSmallBuf;
    //return kLargeBuf;
    if (mPools.empty()) throw runtime_error("No buffer pools defined");
    size_t id    = 0;
    size_t bsize = mPools[id]->buffer_size();
    size_t N = mPools.size();
    for (size_t i=0; i<N; i++) {
	size_t tsize = mPools[i]->buffer_size();
	if (bsize < selid && tsize > bsize) {
	    id = i;
	    bsize = tsize;
	} else if (tsize >= selid && tsize < bsize) {
	    id = i;
	    bsize = tsize;
	}
    }
    return static_cast<bufclass_type>(id);
}

//======================================  Get a buffer
buffer* 
seg_pool_mgr::get_buffer(bufclass_type bc) {
    return mPools[bc]->get_buffer();
}

//======================================  Get a buffer (don't wait)
buffer* 
seg_pool_mgr::get_buffer_now(bufclass_type bc) {
    return mPools[bc]->get_buffer_now();
}
